/*
 * Copyright 2018 Equilinox Mod Kit Project
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *    the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package equilinoxmodkit.experimental;


import org.lwjgl.input.Keyboard;
import toolbox.MyKeyboard;

import java.util.ArrayList;


/**
 * <i>!!! EXPERIMENTAL !!!</i><br>
 * Static class that provides methods to check if keys are pressed and simulate key presses.<br>
 *
 * @author EMK Team
 */
public final class EmkKeyboard {
	
	
	/** Letter keys. */
	public static final int
			A = Keyboard.KEY_A,
			B = Keyboard.KEY_B,
			C = Keyboard.KEY_C,
			D = Keyboard.KEY_D,
			E = Keyboard.KEY_E,
			F = Keyboard.KEY_F,
			G = Keyboard.KEY_G,
			H = Keyboard.KEY_H,
			I = Keyboard.KEY_I,
			J = Keyboard.KEY_J,
			K = Keyboard.KEY_K,
			L = Keyboard.KEY_L,
			M = Keyboard.KEY_M,
			N = Keyboard.KEY_N,
			O = Keyboard.KEY_O,
			P = Keyboard.KEY_P,
			Q = Keyboard.KEY_Q,
			R = Keyboard.KEY_R,
			S = Keyboard.KEY_S,
			T = Keyboard.KEY_T,
			U = Keyboard.KEY_U,
			V = Keyboard.KEY_V,
			W = Keyboard.KEY_W,
			X = Keyboard.KEY_X,
			Y = Keyboard.KEY_Y,
			Z = Keyboard.KEY_Z;
	/** Number keys. */
	public static final int
			NO_1 = Keyboard.KEY_1,
			NO_2 = Keyboard.KEY_2,
			NO_3 = Keyboard.KEY_3,
			NO_4 = Keyboard.KEY_4,
			NO_5 = Keyboard.KEY_5,
			NO_6 = Keyboard.KEY_6,
			NO_7 = Keyboard.KEY_7,
			NO_8 = Keyboard.KEY_8,
			NO_9 = Keyboard.KEY_9,
			NO_0 = Keyboard.KEY_0;
	/** Special printable keys. */
	public static final int
			ESCAPE = Keyboard.KEY_ESCAPE,
			TILDE = Keyboard.KEY_GRAVE,
			TAB = Keyboard.KEY_TAB,
			CAPS_LOCK = Keyboard.KEY_CAPITAL,
			LSHIFT = Keyboard.KEY_LSHIFT,
			LCONTROL = Keyboard.KEY_LCONTROL,
			LSUPER = Keyboard.KEY_LMETA,
			LALT = Keyboard.KEY_LMENU,
			SPACE = Keyboard.KEY_SPACE,
			ALT_GR = Keyboard.KEY_RMENU,
			RSUPER = Keyboard.KEY_RMETA,
			RCONTROL = Keyboard.KEY_RCONTROL,
			RSHIFT = Keyboard.KEY_RSHIFT,
			ENTER = Keyboard.KEY_RETURN,
			BACKSPACE = Keyboard.KEY_BACK,
			COMMA = Keyboard.KEY_COMMA,
			PERIOD = Keyboard.KEY_PERIOD;
	/** Function keys. */
	public static final int
			F1 = Keyboard.KEY_F1,
			F2 = Keyboard.KEY_F2,
			F3 = Keyboard.KEY_F3,
			F4 = Keyboard.KEY_F4,
			F5 = Keyboard.KEY_F5,
			F6 = Keyboard.KEY_F6,
			F7 = Keyboard.KEY_F7,
			F8 = Keyboard.KEY_F8,
			F9 = Keyboard.KEY_F9,
			F10 = Keyboard.KEY_F10,
			F11 = Keyboard.KEY_F11,
			F12 = Keyboard.KEY_F12;
	/** Secondary function keys. */
	public static final int
			INSERT = Keyboard.KEY_INSERT,
			DELETE = Keyboard.KEY_DELETE,
			HOME = Keyboard.KEY_HOME,
			END = Keyboard.KEY_END,
			PAGE_UP = Keyboard.KEY_PRIOR,
			PAGE_DOWN = Keyboard.KEY_NEXT;
	/** Arrow keys. */
	public static final int
			ARROW_UP = Keyboard.KEY_UP,
			ARROW_LEFT = Keyboard.KEY_LEFT,
			ARROW_DOWN = Keyboard.KEY_DOWN,
			ARROW_RIGHT = Keyboard.KEY_RIGHT;
	
	
	private static ArrayList<Integer> simulatedKeyPresses = new ArrayList<>();
	
	
	/** Private constructor because of statically used class. */
	private EmkKeyboard() {
	}
	
	
	/**
	 * Determines whether the key with the specified code is held down.
	 *
	 * @param key Key code to check
	 *
	 * @return {@code True} if {@code key} is held down.
	 */
	public static boolean isKeyDown( int key ) {
		return MyKeyboard.getKeyboard().isKeyDown( key );
	}
	
	/**
	 * Adds the specified key code to a list of simulated key presses that get's injected into
	 * Equilinox' update method next time it executes.
	 *
	 * @param key Key code to add
	 */
	public static void invokeKey( int key ) {
		simulatedKeyPresses.add( key );
	}
	
	
	/**
	 * @return List of simulated key presses added by using {@link #invokeKey(int)}.
	 */
	public static ArrayList<Integer> getSimulatedKeyPresses() {
		return simulatedKeyPresses;
	}
	
	
}
