/*
 * Copyright 2018 Equilinox Mod Kit Project
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *    the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package equilinoxmodkit.experimental;


import toolbox.MyMouse;

import java.util.ArrayList;


/**
 * <i>!!! EXPERIMENTAL !!!</i><br>
 * Static class that provides methods to check if buttons are pressed and simulate button presses.
 *
 * @author EMK Team
 */
public final class EmkMouse {
	
	
	/** Main mouse buttons. */
	public static final int
			BUTTON_LEFT = 0,
			BUTTON_MIDDLE = 1,
			BUTTON_RIGHT = 2;
	
	
	private static ArrayList<Integer> simulatedButtonPresses = new ArrayList<>();
	
	
	/** Private constructor because of statically used class. */
	private EmkMouse() {
	}
	
	
	/**
	 * Determines whether the button with the specified code is held down.
	 *
	 * @param button Button code to check
	 *
	 * @return {@code True} if {@code button} is held down.
	 */
	public static boolean isButtonDown( int button ) {
		switch( button ) {
			case BUTTON_LEFT:
				return MyMouse.getActiveMouse().isLeftButtonDown();
			case BUTTON_MIDDLE:
				return MyMouse.getActiveMouse().isMiddleClick();
			case BUTTON_RIGHT:
				return MyMouse.getActiveMouse().isRightButtonDown();
			default:
				return false;
		}
	}
	
	/**
	 * Adds the specified button code to a list of simulated button presses that get's injected into
	 * Equilinox' update method next time it executes.
	 *
	 * @param button Button code to add
	 */
	public static void invokeButton( int button ) {
		simulatedButtonPresses.add( button );
	}
	
	/**
	 * @return Horizontal position of cursor on screen in pixels.
	 */
	public static float getX() {
		return MyMouse.getActiveMouse().getX();
	}
	
	/**
	 * @return Vertical position of cursor on screen in pixels.
	 */
	public static float getY() {
		return MyMouse.getActiveMouse().getY();
	}
	
	/**
	 * @return Amount the horizontal position of the cursor has changed in pixels.
	 */
	public static int getDX() {
		return MyMouse.getActiveMouse().getDX();
	}
	
	/**
	 * @return Amount the vertical position of the cursor has changed in pixels.
	 */
	public static int getDY() {
		return MyMouse.getActiveMouse().getDY();
	}
	
	/**
	 * @return Number of clicks the mouse wheel has been moved.
	 */
	public static int getDWheel() {
		return MyMouse.getActiveMouse().getDWheel();
	}
	
	
	/**
	 * @return List of simulated button presses added by using {@link #invokeButton(int)}.
	 */
	public static ArrayList<Integer> getSimulatedButtonPresses() {
		return simulatedButtonPresses;
	}
	
	
}
